/*
 *     +--------------------------------------+
 *     |  Version historique du 29 aout 2018  |
 *     +--------------------------------------+
 *
 */

#include  <stdio.h>
#include  <stdlib.h>
#include  <string.h>
#include <fcntl.h>
#include  <float.h>

#define MAXFLOAT FLT_MAX

#include  "floatimg.h"

/* -------------------------------------------------------- */
int create_floatimg(FloatImg *head, int w, int h, int t)
{
int		surface, size;
float		*fptr;

#if DEBUG_LEVEL
fprintf(stderr, ">>> %20s ( %p %d %d )\n", __func__, head, w, h);
#endif

memset(head, 0, sizeof(FloatImg));

surface = w * h;
size    = surface * 3 * sizeof(float);			

fptr = (float *)malloc(size);
if (NULL==fptr) {
	fprintf(stderr, "%s : nom mem, exiting.\n", __func__);
	exit(1);
	}

#if DEBUG_LEVEL > 1
fprintf(stderr, "    got %d bytes at %p\n", size, fptr);
#endif

head->width = w;	head->height = h;
head->R = fptr;	
head->G = fptr + surface;
head->B = fptr + surface + surface;

return 0;
}

/* -------------------------------------------------------- */
int print_floatimg(FloatImg *head, char *txt)
{
printf("--- %s at %p ---\n", txt, head);

printf("    size      %d x %d\n", head->width, head->height);
printf("    pixels@  %p  %p  %p\n", head->R, head->G, head->B);

return 0;
}
/* -------------------------------------------------------- */
static float getmax_redvalue(FloatImg *head)
{
float		maxval;
int		foo;

maxval = 0.0;

for (foo=0; foo<(head->width*head->height); foo++) {
	if (head->R[foo] > maxval)	maxval = head->R[foo];
	}

return maxval;
}
/* -------------------------------------------------------- */
static float getmax_allvalues(FloatImg *head)
{
float		maxval;
int		foo;

maxval = 0.0;		/* no negative values allowed */

for (foo=0; foo<(head->width*head->height); foo++) {
	if (head->R[foo] > maxval) maxval = head->R[foo];
	if (head->G[foo] > maxval) maxval = head->G[foo];
	if (head->B[foo] > maxval) maxval = head->B[foo];
	}

return maxval;
}
/* -------------------------------------------------------- */
static float getmin_allvalues(FloatImg *head)
{
float		minval;
int		foo;

minval = MAXFLOAT;		/* lol */

for (foo=0; foo<(head->width*head->height); foo++) {
	if (head->R[foo] < minval) minval = head->R[foo];
	if (head->G[foo] < minval) minval = head->G[foo];
	if (head->B[foo] < minval) minval = head->B[foo];
	}

return minval;
}

/* -------------------------------------------------------- */
/*
 *	we save only the red channel
 */
int save_floatimg_as_pgm(char *fname, FloatImg *head)
{
FILE		*fp;
float		maximum, fk;
int		idx, sz, value, foo;

#if DEBUG_LEVEL
fprintf(stderr, ">>> %20s ( '%s' %p )\n", __func__, fname, head);
#endif

if (NULL==(fp=fopen(fname, "w"))) {
	perror(fname);
	return -1;
	}

fprintf(fp, "P2\n%d %d\n", head->width, head->height);

maximum = getmax_redvalue(head);
fprintf(fp, "# max value %f\n", maximum);
fk = maximum / 65536.0;
fprintf(fp, "# fk = %f\n", fk);
fprintf(fp, "65535\n");

sz = head->width*head->height;
foo = 0;
for (idx=0; idx<sz; idx++) {
	if (fk > 0)	value = (int)(head->R[idx] / fk);
	else		value = 0;
	foo += fprintf(fp, "%d", value);
	if (foo > 70) {
		fputs("\n", fp); 	foo = 0;
		}
	else	{
		fputs(" ", fp);		foo++;
		}
	}
fputs("\n", fp);
fclose(fp);
return 0;
}
/* -------------------------------------------------------- */

int save_floatimg_as_pnm(char *fname, FloatImg *head)
{
FILE		*fp;
float		maximum, fk;
int		idx, sz, Rv, Gv, Bv, foo;

#if DEBUG_LEVEL
fprintf(stderr, ">>> %20s ( '%s' %p )\n", __func__, fname, head);
#endif

if (NULL==(fp=fopen(fname, "w"))) {
	perror(fname);
	return -1;
	}

fprintf(fp, "P3\n%d %d\n", head->width, head->height);

maximum = getmax_allvalues(head);
fprintf(fp, "# max value %f\n", maximum);
fk = maximum / 65536.0;
fprintf(fp, "# fk = %f\n", fk);
fprintf(fp, "65535\n");

sz = head->width*head->height;
foo = 0;
for (idx=0; idx<sz; idx++) {
	if (fk > 0) {
		Rv = (int)(head->R[idx] / fk);
		Gv = (int)(head->G[idx] / fk);
		Bv = (int)(head->B[idx] / fk);
		}
	else	{
		Rv = Gv = Bv = 0;
		}
	foo += fprintf(fp, "%d %d %d", Rv, Gv, Bv);
	if (foo > 65) {
		fputs("\n", fp); 	foo = 0;
		}
	else	{
		fputs(" ", fp);		foo++;
		}
	}
fputs("\n", fp);
fclose(fp);
return 0;
}

/* -------------------------------------------------------- */

int clear_floatimg(FloatImg *head)
{
int		size;

#if DEBUG_LEVEL
fprintf(stderr, ">>> %20s ( %p )\n", __func__, head);
#endif

size = head->width * head->height * 3 * sizeof(float);
memset(head->R, 0, size);

return -1;
}
/* -------------------------------------------------------- */
int dump_floatimg(char *fname, FloatImg *head)
{
FILE	*fp;
int	foo, nbre, dims[3];

#if DEBUG_LEVEL
fprintf(stderr, ">>> %20s ( '%s' %p )\n", __func__, fname, head);
#endif

fp = fopen(fname, "w");
if (NULL==fp) {
	perror(fname);
	return -1;
	}

dims[0] = head->width;	dims[1] = head->height;
dims[2] = 0;

foo = fwrite(dims, sizeof(int), 3, fp);
if (3 != foo) {
	perror(fname);
	fclose(fp);
	return -2;
	}
nbre = head->width*head->height*3;
#if DEBUG_LEVEL
fprintf(stderr, "%s : data at %p\n", __func__, head->R);
#endif
foo = fwrite(head->R, sizeof(float), nbre, fp);
if (nbre!=foo) {
	perror(fname);
	fclose(fp);
	return -3;
	}
	
fclose(fp);

return 0;
}
/* -------------------------------------------------------- */
int get_floatimg_dims(char *fname, int *pw, int *ph)
{
FILE	*fp;
int	dims[3];

fp = fopen(fname, "r");
if (NULL==fp) {
	perror(fname);
	return -1;
	}
if (3 != fread(dims, sizeof(int), 3, fp)) {
	fprintf(stderr, "%s : short read\n", fname);
	fclose(fp);
	return -1;
	}

*pw = dims[0];		*ph = dims[1];

fclose(fp);
return 0;
}
/* -------------------------------------------------------- */
int load_floatimg(char *fname, FloatImg *head)
{
FILE	*fp;
// int	foo, nbre, dims[3];

fp = fopen(fname, "r");
if (NULL==fp) {
	perror(fname);
	exit(1);
	}
	

fclose(fp);
	
return -1;
}
/* -------------------------------------------------------- */
int create_floatimg_from_dump(char *fname, FloatImg *head)
{
FILE	*fp;
int	foo, dims[3];

#if DEBUG_LEVEL
fprintf(stderr, ">>> %20s ( '%s' %p )\n", __func__, fname, head);
#endif

fp = fopen(fname, "r");
if (NULL==fp) {
	perror(fname);
	exit(1);
	}
	
foo = fread(dims, sizeof(int), 3, fp);  
if (3 != foo) {
	fprintf(stderr, "%s : short read\n", fname);
	fclose(fp);
	return -1;
	}
#if DEBUG_LEVEL
fprintf(stderr, "%s : got [ %d %d %d ] from %s\n", __func__,
			dims[0], dims[1], dims[2], fname);
#endif

create_floatimg(head, dims[0], dims[1], dims[2]); 

foo = fread(head->R, sizeof(float), dims[0]*dims[1]*3, fp);

fclose(fp);
	
return -1;
}
/* -------------------------------------------------------- */
int plot_floatimg(FloatImg *head, int x, int y, float gray)
{
int	offset;

offset = x + (y * head->width);

#if DEBUG_LEVEL > 1
fprintf(stderr, ">>> %s ( %p %d %d %f )\n", __func__, head, x, y, gray);
fprintf(stderr, "    offset %d\n", offset);
#endif

head->R[offset] = gray;

return 0;
}
/* -------------------------------------------------------- */
int plot_rgb_floatimg(FloatImg *head, int x, int y, 
					float r, float g, float b)
{
int	offset;

offset = x + (y * head->width);

#if DEBUG_LEVEL > 1
fprintf(stderr, ">>> %s ( %p %d %d %f )\n", __func__, head, x, y, gray);
fprintf(stderr, "    offset %d\n", offset);
#endif

head->R[offset] = r;
head->G[offset] = g;
head->B[offset] = b;

return 0;
}
/* -------------------------------------------------------- */
int	add_to_fimg_Rpix(FloatImg *head, int x, int y, float v)
{
int	offset;
offset = x + (y * head->width);
head->R[offset] += v;
return 0;
}
/* -------------------------------------------------------- */
int	add_to_fimg_rgb(FloatImg *head, int x, int y,
					float r, float g, float b)
{
int	offset;
offset = x + (y * head->width);
head->R[offset] += r;
head->G[offset] += g;
head->B[offset] += b;
return 0;
}

/* -------------------------------------------------------- */
